/*
 * TT's Plugin Support.cp
 *
 * Support code for TT's Plugin Starter and other plugins:
 *
 *   Provides Startup and Termination handling for both Mac OS and Windows
 *
 * Brought to you for free by Thomas Tempelmann, http://www.tempel.org/rb/
 */

#if TARGET_OS_WIN32
	#define log(x) OutputDebugString(x)
#else
	#include <MacMemory.h>
	#include "dcon.h"
	#define log(x) dprintf(x)
#endif

#if TARGET_OS_MAC and not TARGET_CARBON
	#include "QuitHandler.h"
#endif

#include "TT's Plugin Support.h"


	// forward declarations

void InitTTsPluginSupport ();



#if TARGET_OS_WIN32
	// this is a Windows-only feature: We can get notified when our runtime environment starts and stops
	extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID fImpLoad);
	extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID fImpLoad)
	{
	    switch(fdwReason) {
	        case DLL_PROCESS_ATTACH: {
	            // A process has connected to the DLL.
	            // For this plugin, this means that the app has just started
	            
				//log ("[TT's Plugin Starter] DLL_PROCESS_ATTACH\n");
	            break;
	        }

	        case DLL_PROCESS_DETACH: {
	            // A process has released our DLL
	            // For this plugin, this means that the app is terminating
				
				//log ("[TT's Plugin Starter] DLL_PROCESS_DETACH\n");
	            break;
	        }
	    }

	    return TRUE;
	}
#endif


// To initialize the static C++ objects, we may need to call a special routine
// that the linker will generate. Here's the preparation for this:
#if TARGET_OS_WIN32
	extern "C" far void _RunInit(void);
	#define __InitCode__ _RunInit
#elif TARGET_CPU_PPC
	extern "C" void __sinit(void);
	#define __InitCode__ __sinit
#elif TARGET_CPU_68K
	extern "C" far void __InitCode__ (void);
#endif
extern "C" void __destroy_global_chain (void);


static long gGlobalsInitedSig = 0;

static void quitHandler (void)
{
	//log ("[TT's Plugin Starter] quitHandler called\n");

	// dispose of all global C++ objects (calls their destructors)
	if (gGlobalsInitedSig == '<ok>') {
		gGlobalsInitedSig = 0;
		__destroy_global_chain ();
	}
}


	/*
	 * The following code shows how static con- and destructors get invoked.
	 *
	 * The result is this:
	 *   Under Windows, static constructors get called first, even before
	 * "PluginEntry" gets called. Static destructors are called at time
	 * the app terminates.
	 *   Under Mac OS, static constructors get called first, just as in
	 * Windows, but the destructors are _not_ called at program termination.
	 * However, this plugin support code installs a quit handler that takes
	 * care of the proper destruction of objects by manually calling the
	 * appropriate function.
	 */

static class globalsInitTracer
{
public:
	globalsInitTracer();
	~globalsInitTracer();
} globalsInitTracerObj;

globalsInitTracer::globalsInitTracer()
{
	//log ("[TT's Plugin Starter] constructing globalsInitTracerObj\n");
	gGlobalsInitedSig = '<ok>';
	InitTTsPluginSupport ();
	PluginInit ();
}

globalsInitTracer::~globalsInitTracer()
{
	//log ("[TT's Plugin Starter] destructing globalsInitTracerObj\n");
	PluginExit ();
}

static Boolean gPluginInited = false;

void InitTTsPluginSupport ()
{
	if (not gPluginInited) {

		//log ("[TT's Plugin Starter] InitTTsPluginSupport called\n");

		gPluginInited = true;
		
		if (gGlobalsInitedSig != '<ok>') {
			// oops - RB's runtime forgot to init us - let's catch up now ourselves:
			log ("[TT's Plugin Starter] Error: Globals did not get instantiated as expected.\n");
			__InitCode__ ();	// calls constructors for static and global C++ objects
		}
		
		#if TARGET_OS_MAC and not TARGET_CARBON
		
			// We want to get notified if the process ends that was running when
			// this plugin got initialized. Then, we can de-init the plugin again.
			// (We don't need this for Carbon and: the class destructor there
			// gets called already by other ways):
			InstallQuitHandler (quitHandler);
		
		#endif
	}
}
	
// EOF
