/*
	FileFromPath() XFCN v1.2
	
	1991 Apple Computer, Inc.; by Mike Byrne
	
	This XFCN will take a full pathname and simply return the filename at the end of it.
	
	Form:
	FileFromPath(<pathName>)
	
	# the MPW 3.2 build commands:
	C -b FileFromPath.c -mbg off
		Link -w -t STAK -c WILD -rt XFCN=612 
			-m ENTRYPOINT 
			-sg FileFromPath 
			FileFromPath.c.o 
			"{Libraries}HyperXLib.o" 
			"{Libraries}Runtime.o" 
			"{Libraries}Interface.o" 
			"{CLibraries}StdCLib.o" 
			-o "teststack"
*/

#include <string.h>
#include <Memory.h>
#include "HyperXCmd.h"

#define NULL (long) 0
#define NIL (long) 0

#define kNumParams 1


/* prototypes */
void ErrorBack(XCmdPtr paramPtr, char *message);
void MoveLockParams ( XCmdPtr paramPtr, short paramCount );
void UnlockParams  ( XCmdPtr paramPtr, short paramCount );



pascal void EntryPoint(XCmdPtr paramPtr)
{
	/* variable declarations */
	char*	fileName;


	/* move high and lock the parameters. */
	MoveLockParams(paramPtr, paramPtr->paramCount);

	/* check for copyright or syntax help request */
	if (!strcmp( (char*)*paramPtr->params[0], "!") ) {
		ErrorBack(paramPtr, "v1.2, 1991 Apple Computer, Inc.; by Mike Byrne");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	} else if (!strcmp ( (char*)*paramPtr->params[0], "?") ) {
		ErrorBack(paramPtr, "FileFromPath syntax is 'FileFromPath(<pathName>)'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}

	/* not a copyright or help request.       */     
	/* check for correct number of parameters */
	if (paramPtr->paramCount != kNumParams) {
		ErrorBack(paramPtr, "Error: FileFromPath syntax is 'FileFromPath(<pathName>)'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}
	
	/* do a strrchr to find the last colon, then increment by 1 to skip that. */
	fileName =  strrchr ((char*) *paramPtr->params[0], ':');
	fileName++;
	
	/* that's it.  go home. */
	ErrorBack(paramPtr, fileName);
	UnlockParams(paramPtr, paramPtr->paramCount);
	return;

}





	
/* allocate and load up paramPtr->returnValue with a string 
   -------------------------------------------------------- */
void ErrorBack(XCmdPtr paramPtr, char *message)
{
	Handle  mesHnd;

	/*
		Allocate space for an error message.
		Copy the string into it.
		Return the handle to HyperCard.
	*/
	mesHnd = NewHandle((long)(strlen(message)+1));
	if (mesHnd == nil) return;
	strcpy((char *)*mesHnd,message);
	paramPtr->returnValue = mesHnd;
}



/*  move high and lock down all parameters  
    ----------------------------------------------------------------------- */
void MoveLockParams ( XCmdPtr paramPtr, short paramCount )
{
	short i;
	
	for(i=0; i <= paramCount-1; i++)
	{
		MoveHHi(paramPtr->params[i]);
		HLock(paramPtr->params[i]);
	}
}




/* unlock all parameter handles in the XCmdBlock  
   ---------------------------------------------  */
void UnlockParams  ( XCmdPtr paramPtr, short paramCount )
{	short i;
	
	for(i=0; i <= paramCount-1; i++)
		{ HUnlock(paramPtr->params[i]);}
}
