/*
	GetSpeakerVol() XFCN v1.1
	
	1991 Apple Computer, Inc.; by Mike Byrne
	
	This XFCN takes no parameters.  It simply returns the number of the speaker setting, as it should
	say in the Control Panel.
	
	Form:
	GetSpeakerVol()
	
	# the MPW 3.2 build commands:
	C -b GetSpeakerVol.c -mbg off
		Link -w -t STAK -c WILD -rt XFCN=610 
			-m ENTRYPOINT 
			-sg GetSpeakerVol 
			GetSpeakerVol.c.o 
			"{Libraries}HyperXLib.o" 
			"{Libraries}Runtime.o" 
			"{Libraries}Interface.o" 
			"{CLibraries}StdCLib.o" 
			-o "teststack"
*/

#include <Types.h>
#include <Sound.h>
#include <string.h>
#include <Memory.h>
#include "HyperXCmd.h"

#define NULL (long) 0
#define NIL (long) 0

#define kNumParams 0


/* prototypes */
void ErrorBack(XCmdPtr paramPtr, char *message);
void MoveLockParams ( XCmdPtr paramPtr, short paramCount );
void UnlockParams  ( XCmdPtr paramPtr, short paramCount );



pascal void EntryPoint(XCmdPtr paramPtr)
{
	/* variable declarations */
	short	volNum;
	Str255	retString;


	/* move high and lock the parameters. */
	MoveLockParams(paramPtr, paramPtr->paramCount);

	/* check for copyright or syntax help request */
	if (!strcmp( (char*)*paramPtr->params[0], "!") ) {
		ErrorBack(paramPtr, "v1.1, 1991 Apple Computer, Inc.; by Mike Byrne");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	} else if (!strcmp ( (char*)*paramPtr->params[0], "?") ) {
		ErrorBack(paramPtr, "GetSpeakerVol syntax is 'GetSpeakerVol()'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}

	/* not a copyright or help request.       */     
	/* check for correct number of parameters */
	if (paramPtr->paramCount != kNumParams) {
		ErrorBack(paramPtr, "Error: GetSpeakerVol syntax is 'GetSpeakerVol()'");
		UnlockParams(paramPtr, paramPtr->paramCount);
		return;
	}

	/* okay, now the meat.  Send in the call. */
	GetSoundVol(&volNum);
	NumToString( (long) volNum, retString);
	p2cstr(retString);
	ErrorBack(paramPtr, retString);
	return;

}





	
/* allocate and load up paramPtr->returnValue with a string 
   -------------------------------------------------------- */
void ErrorBack(XCmdPtr paramPtr, char *message)
{
	Handle  mesHnd;

	/*
		Allocate space for an error message.
		Copy the string into it.
		Return the handle to HyperCard.
	*/
	mesHnd = NewHandle((long)(strlen(message)+1));
	if (mesHnd == nil) return;
	strcpy((char *)*mesHnd,message);
	paramPtr->returnValue = mesHnd;
}



/*  move high and lock down all parameters  
    ----------------------------------------------------------------------- */
void MoveLockParams ( XCmdPtr paramPtr, short paramCount )
{
	short i;
	
	for(i=0; i <= paramCount-1; i++)
	{
		MoveHHi(paramPtr->params[i]);
		HLock(paramPtr->params[i]);
	}
}




/* unlock all parameter handles in the XCmdBlock  
   ---------------------------------------------  */
void UnlockParams  ( XCmdPtr paramPtr, short paramCount )
{	short i;
	
	for(i=0; i <= paramCount-1; i++)
		{ HUnlock(paramPtr->params[i]);}
}
